{****************************************************************************
  FILE..........: TL30P.PAS
  COPYRIGHT.....: Copyright(c) 1995, Bennet-Tec Information Systems, Inc.
  VERSION.......: 3.00
  DESCRIPTION...: TTList class
 ****************************************************************************}

unit TL30p;

interface

uses SysUtils, WinTypes, WinProcs, Classes, Graphics, Forms, Controls,
     VBXCtrl, BIVBX;

{ $DEFINE InitTTList}
{^- Remove space to enable default init data -- See documentation. }

type
  TTListExpandEvent = procedure (Sender: TObject; var I: Integer) of object;
  TTListCollapseEvent = procedure (Sender: TObject; var I: Integer) of
    object;
  TTListPictureClickEvent = procedure (Sender: TObject) of object;
  TTListPictureDblClickEvent = procedure (Sender: TObject) of object;
  TTListClickEvent = procedure (Sender: TObject) of object;
  TTListDblClickEvent = procedure (Sender: TObject) of object;
  TTListKeyDownEvent = procedure (Sender: TObject; var KeyCode: Integer; var
    Shift: Integer) of object;
  TTListKeyUpEvent = procedure (Sender: TObject; var KeyCode: Integer; var
    Shift: Integer) of object;
  TTListKeyPressEvent = procedure (Sender: TObject; var KeyAscii: Integer) of
    object;
  TTListDragDropExEvent = procedure (Sender: TObject; var hWnd: Integer; var
    X: Integer; var Y: Integer) of object;
  TTListDragOverExEvent = procedure (Sender: TObject; var hWnd: Integer; var
    X: Integer; var Y: Integer; var State: Integer) of object;
  TTListHScrollEvent = procedure (Sender: TObject; var CancelDefault: Integer;
    var ScrollCode: Integer; var Pos: Integer) of object;
  TTListVScrollEvent = procedure (Sender: TObject; var CancelDefault: Integer;
    var ScrollCode: Integer; var Pos: Integer) of object;
  TTListPlusMinusClickEvent = procedure (Sender: TObject; var I: Integer) of
    object;
  TTListPlusMinusDblClickEvent = procedure (Sender: TObject; var I: Integer)
    of object;
  TTListMarkClickEvent = procedure (Sender: TObject; var I: Integer) of
    object;
  TTListMarkDblClickEvent = procedure (Sender: TObject; var I: Integer) of
    object;
  TTListRequestEditingEvent = procedure (Sender: TObject; var Cancel: Integer;
    var ItemIndex: Integer; var TextToEdit: TBasicString; var Options: Integer)
    of object;
  TTListAfterEditingEvent = procedure (Sender: TObject; var ItemIndex:
    Integer; var EditedText: TBasicString; var CancelledBy: Integer) of
    object;
  TTListEditingKeyDownEvent = procedure (Sender: TObject; var ItemIndex:
    Integer; var KeyCode: Integer; var Shift: Integer) of object;
  TTListEditingKeyUpEvent = procedure (Sender: TObject; var ItemIndex:
    Integer; var KeyCode: Integer; var Shift: Integer) of object;
  TTListEditingKeyPressEvent = procedure (Sender: TObject; var ItemIndex:
    Integer; var KeyAscii: Integer) of object;

  { TTList }

  TTList = class(TVBXControl)
  protected
    FOnExpand: TTListExpandEvent;
    FOnCollapse: TTListCollapseEvent;
    FOnPictureClick: TTListPictureClickEvent;
    FOnPictureDblClick: TTListPictureDblClickEvent;
    FOnClick: TTListClickEvent;
    FOnDblClick: TTListDblClickEvent;
    FOnKeyDown: TTListKeyDownEvent;
    FOnKeyUp: TTListKeyUpEvent;
    FOnKeyPress: TTListKeyPressEvent;
    FOnDragDropEx: TTListDragDropExEvent;
    FOnDragOverEx: TTListDragOverExEvent;
    FOnHScroll: TTListHScrollEvent;
    FOnVScroll: TTListVScrollEvent;
    FOnPlusMinusClick: TTListPlusMinusClickEvent;
    FOnPlusMinusDblClick: TTListPlusMinusDblClickEvent;
    FOnMarkClick: TTListMarkClickEvent;
    FOnMarkDblClick: TTListMarkDblClickEvent;
    FOnRequestEditing: TTListRequestEditingEvent;
    FOnAfterEditing: TTListAfterEditingEvent;
    FOnEditingKeyDown: TTListEditingKeyDownEvent;
    FOnEditingKeyUp: TTListEditingKeyUpEvent;
    FOnEditingKeyPress: TTListEditingKeyPressEvent;
    { Event handler }
    procedure HandleVBXEvent(var Message: TWMVBXFireEvent); override;
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(AOwner: TComponent); override;

    property Index: TVBInteger index 1 read GetIntProp;
    property ListCount: TVBInteger index 32 read GetIntProp;
    property ListIndex: TVBInteger index 33 read GetIntProp write SetIntProp;
    property Redraw: Boolean index 34 read GetBoolProp write SetBoolProp;
    property Selected[Index: Integer]: TVBInteger index 37 read
      GetIntArrayProp write SetIntArrayProp;
    property List[Index: Integer]: TVBString index 38 read GetStringArrayProp
      write SetStringArrayProp;
    property Image[Index: Integer]: TVBPic index 39 read GetPictureArrayProp
      write SetPictureArrayProp;
    property InvImage[Index: Integer]: TVBPic index 40 read
      GetPictureArrayProp write SetPictureArrayProp;
    property ItemBackColor[Index: Integer]: TColor index 41 read
      GetColorArrayProp write SetColorArrayProp;
    property ItemForeColor[Index: Integer]: TColor index 42 read
      GetColorArrayProp write SetColorArrayProp;
    property ItemFontName[Index: Integer]: TVBString index 43 read
      GetStringArrayProp write SetStringArrayProp;
    property ItemFontSize[Index: Integer]: TVBInteger index 44 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemFontBold[Index: Integer]: TVBInteger index 45 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemFontItalic[Index: Integer]: TVBInteger index 46 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemFontStrike[Index: Integer]: TVBInteger index 47 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemFontUnder[Index: Integer]: TVBInteger index 48 read
      GetIntArrayProp write SetIntArrayProp;
    property Shift[Index: Integer]: TVBInteger index 49 read GetIntArrayProp
      write SetIntArrayProp;
    property FullPath[Index: Integer]: TVBString index 50 read
      GetStringArrayProp;
    property Expand[Index: Integer]: TVBInteger index 52 read GetIntArrayProp
      write SetIntArrayProp;
    property TopIndex: TVBInteger index 54 read GetIntProp write SetIntProp;
    property HasSubItems[Index: Integer]: TVBInteger index 55 read
      GetIntArrayProp;
    property IsItemVisible[Index: Integer]: TVBInteger index 56 read
      GetIntArrayProp;
    property InsertItemEx[Index: Integer]: TVBString index 65 write
      SetStringArrayProp;
    property ItemMultiLine[Index: Integer]: TVBInteger index 69 read
      GetIntArrayProp write SetIntArrayProp;
    property CurrentItem: TVBString index 70 read GetStringProp write
      SetStringProp;
    property CoerceIndex[Index: Integer]: TVBInteger index 73 read
      GetIntArrayProp write SetIntArrayProp;
    property DropTarget: TVBInteger index 74 read GetIntProp;
    property ItemType[Index: Integer]: TVBInteger index 75 read
      GetIntArrayProp;
    property ItemIntValue[Index: Integer]: TVBInteger index 76 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemLngValue[Index: Integer]: TVBLongint index 77 read
      GetLongArrayProp write SetLongArrayProp;
    property ItemSngValue[Index: Integer]: Single index 78 read
      GetSingleArrayProp write SetSingleArrayProp;
    property ItemStrValue[Index: Integer]: TVBString index 79 read
      GetStringArrayProp write SetStringArrayProp;
    property Add[Index: Integer]: TVBLongint index 80 write SetLongArrayProp;
    property Insert[Index: Integer]: TVBLongint index 81 write
      SetLongArrayProp;
    property CopyItem[Index: Integer]: TVBLongint index 82 read
      GetLongArrayProp;
    property CopyItemSub[Index: Integer]: TVBLongint index 83 read
      GetLongArrayProp;
    property CopySelected: TVBLongint index 84 read GetLongProp;
    property ClearItem: TVBInteger index 85 write SetIntProp;
    property TriggerEvents: TVBLongint index 90 read GetLongProp write
      SetProp;
    property DragEx: TVBEnum index 92 write SetEnumProp;
    property BottomIndex: TVBInteger index 95 read GetIntProp;
    property ScrollHorz: TVBInteger index 96 read GetIntProp write SetIntProp;
    property ItemEditText[Index: Integer]: TVBInteger index 97 write
      SetIntArrayProp;
    property LoadAndAdd[Index: Integer]: TVBInteger index 98 write
      SetIntArrayProp;
    property LoadAndInsert[Index: Integer]: TVBInteger index 99 write
      SetIntArrayProp;
    property Save[Index: Integer]: TVBInteger index 100 write SetIntArrayProp;
    property SaveSub[Index: Integer]: TVBInteger index 101 write
      SetIntArrayProp;
    property ItemPicValue[Index: Integer]: TVBPic index 102 read
      GetPictureArrayProp write SetPictureArrayProp;
    property Text: TVBString index 103 read GetStringProp write SetStringProp;
    property Clipboard[Index: Integer]: TVBInteger index 104 write
      SetIntArrayProp;
    property IsClipboardAvailable[Index: Integer]: TVBInteger index 105 read
      GetIntArrayProp;
    property ItemMark[Index: Integer]: TVBInteger index 112 read
      GetIntArrayProp write SetIntArrayProp;
    property MarkPicture[Index: Integer]: TVBPic index 113 read
      GetPictureArrayProp write SetPictureArrayProp;
    property MarkTag[Index: Integer]: TVBString index 114 read
      GetStringArrayProp write SetStringArrayProp;
    property TitlePicture[Index: Integer]: TVBPic index 124 read
      GetPictureArrayProp write SetPictureArrayProp;
    property TitleText[Index: Integer]: TVBString index 125 read
      GetStringArrayProp write SetStringArrayProp;
    property TitleVisible[Index: Integer]: TVBInteger index 126 read
      GetIntArrayProp write SetIntArrayProp;
    property TitleWidth[Index: Integer]: TVBInteger index 127 read
      GetIntArrayProp write SetIntArrayProp;
    property Indent[Index: Integer]: TVBInteger index 130 read
      GetIntArrayProp write SetIntArrayProp;
    property ListCountEx[Index: Integer]: TVBInteger index 132 read
      GetIntArrayProp;
    property SelectEx[Index: Integer]: TVBInteger index 133 read
      GetIntArrayProp write SetIntArrayProp;
    property ExpandEx[Index: Integer]: TVBInteger index 134 read
      GetIntArrayProp write SetIntArrayProp;
    property SelItemCount: TVBInteger index 135 read GetIntProp;
    property SelItemIndex[Index: Integer]: TVBInteger index 136 read
      GetIntArrayProp;
    property ItemBM[Index: Integer]: TVBLongint index 137 read
      GetLongArrayProp;
    property CurrentItemBM: TVBLongint index 138 read GetLongProp write
      SetProp;
    property NewIndex: TVBInteger index 139 read GetIntProp;
    property pFile[Index: Integer]: TVBInteger index 141 write
      SetIntArrayProp;
    property CopyOne[Index: Integer]: TVBLongint index 144 read
      GetLongArrayProp;
    property SaveOne[Index: Integer]: TVBInteger index 145 write
      SetIntArrayProp;
    property ItemSorted[Index: Integer]: TVBInteger index 150 read
      GetIntArrayProp write SetIntArrayProp;
    property ItemParentBM[Index: Integer]: TVBLongint index 151 read
      GetLongArrayProp;
    property ItemPMPicType[Index: Integer]: TVBInteger index 153 read
      GetIntArrayProp write SetIntArrayProp;
  published
    property Font;
    property ParentFont;
    property ForeColor: TColor index 16 read GetColorProp write SetColorProp;
    property BackColor: TColor index 17 read GetColorProp write SetColorProp;
    property BorderStyle: TVBEnum index 18 read GetEnumProp write SetEnumProp
      default 1;
    property DragCursor;
    property DragMode;
    property Enabled;
    property MousePointer: TVBEnum index 22 read GetEnumProp write
      SetEnumProp default 0;
    property TabOrder;
    property Visible;
    property ImageStretch: Boolean index 25 read GetBoolProp write
      SetBoolProp default False;
    property InvStyle: TVBEnum index 27 read GetEnumProp write SetEnumProp
      default 0;
    property ViewStyle: TVBEnum index 28 read GetEnumProp write SetEnumProp
      default 0;
    property ShiftStep: TVBInteger index 29 read GetIntProp write SetIntProp;
    property ItemImageDefHeight: TVBInteger index 30 read GetIntProp write
      SetIntProp default 15;
    property ItemImageDefWidth: TVBInteger index 31 read GetIntProp write
      SetIntProp default 15;
    property About: TVBString index 35 {NoRT}read GetStringProp {NoRT}write
      SetStringProp stored False;
    property MultiSelect: TVBEnum index 36 read GetEnumProp {NoRT}write
      SetEnumProp default 0;
    property PictureType: TVBEnum index 51 read GetEnumProp write SetEnumProp
      default 0;
    property PathSeparator: TVBString index 53 read GetStringProp write
      SetStringProp;
    property PictureRoot: TVBPic index 57 read GetPictureProp write
      SetPictureProp stored False;
    property PictureOpen: TVBPic index 58 read GetPictureProp write
      SetPictureProp stored False;
    property PictureClosed: TVBPic index 59 read GetPictureProp write
      SetPictureProp stored False;
    property PictureLeaf: TVBPic index 60 read GetPictureProp write
      SetPictureProp stored False;
    property PictureInverted: TVBPic index 61 read GetPictureProp write
      SetPictureProp stored False;
    property CurrentIndexMethod: TVBEnum index 62 read GetEnumProp write
      SetEnumProp default 0;
    property SelForeColor: TColor index 63 read GetColorProp write
      SetColorProp;
    property SelBackColor: TColor index 64 read GetColorProp write
      SetColorProp;
    property NoIntegralHeight: Boolean index 66 read GetBoolProp {NoRT}write
      SetBoolProp default True;
    property DisableNoScroll: Boolean index 67 read GetBoolProp {NoRT}write
      SetBoolProp default False;
    property WidthOfText: TVBInteger index 68 read GetIntProp write
      SetIntProp;
    property DragHighlight: TVBEnum index 71 read GetEnumProp write
      SetEnumProp default 1;
    property ShowChildren: Boolean index 72 read GetBoolProp write
      SetBoolProp default True;
    property TabStop;
    property FixedSize: Boolean index 88 read GetBoolProp {NoRT}write
      SetBoolProp default False;
    property TabStopDistance: TVBInteger index 89 read GetIntProp write
      SetIntProp;
    property Environment: TVBEnum index 91 read GetEnumProp write SetEnumProp
      default 0;
    property DragIconEx: TVBPic index 93 read GetPictureProp write
      SetPictureProp stored False;
    property Scrollbars: TVBEnum index 94 read GetEnumProp {NoRT}write
      SetEnumProp default 3;
    property PicturePlus: TVBPic index 106 read GetPictureProp write
      SetPictureProp stored False;
    property PictureMinus: TVBPic index 107 read GetPictureProp write
      SetPictureProp stored False;
    property PictureMark: TVBPic index 108 read GetPictureProp write
      SetPictureProp stored False;
    property AutoScrDuringDragDrop: TVBEnum index 109 read GetEnumProp write
      SetEnumProp default 3;
    property NoPictureRoot: Boolean index 110 read GetBoolProp write
      SetBoolProp default False;
    property ViewStyleEx: TVBEnum index 111 read GetEnumProp write
      SetEnumProp default 0;
    property MarkHeight: TVBInteger index 115 read GetIntProp write
      SetIntProp default 0;
    property MarkWidth: TVBInteger index 116 read GetIntProp write SetIntProp
      default 0;
    property AutoExpand: TVBEnum index 117 read GetEnumProp write SetEnumProp
      default 0;
    property TreeLinesStyle: TVBEnum index 119 read GetEnumProp write
      SetEnumProp default 0;
    property PicInMultiLine: TVBEnum index 120 read GetEnumProp write
      SetEnumProp default 1;
    property ShowCaption: TVBEnum index 121 read GetEnumProp write
      SetEnumProp default 0;
    property Caption: TVBString index 122 read GetStringProp write
      SetStringProp;
    property ShowTitles: TVBEnum index 123 read GetEnumProp write SetEnumProp
      default 0;
    property TitleHeight: TVBInteger index 128 read GetIntProp write
      SetIntProp;
    property XOffset: TVBInteger index 129 read GetIntProp write SetIntProp
      default 0;
    property MSOutlineAdd: Boolean index 131 read GetBoolProp write
      SetBoolProp default False;
    property BackwardCompatibility: Boolean index 140 read GetBoolProp write
      SetBoolProp default True;
    property ExpandChildren: Boolean index 142 read GetBoolProp write
      SetBoolProp default False;
    property ExpandNewItem: Boolean index 143 read GetBoolProp write
      SetBoolProp default False;
    property DragModeEx: TVBEnum index 146 read GetEnumProp write SetEnumProp
      default 0;
    property DefMultiLine: Boolean index 152 read GetBoolProp write
      SetBoolProp default False;
    property WidthOfTextMin: TVBInteger index 154 read GetIntProp write
      SetIntProp;
    property SmartDragDrop: Boolean index 155 read GetBoolProp write
      SetBoolProp default False;
    property TreeLinesColor: TColor index 156 read GetColorProp write
      SetColorProp;
    property DrawFocusRect: Boolean index 157 read GetBoolProp write
      SetBoolProp default True;
    property ExplorerCompatible: TVBEnum index 159 read GetEnumProp write
      SetEnumProp default 0;
    property ToolTipsMode: TVBEnum index 160 read GetEnumProp write
      SetEnumProp default 0;
    property ToolTipsViewStyle: TVBEnum index 161 read GetEnumProp write
      SetEnumProp default 0;
    property ToolTipsForeColor: TColor index 162 read GetColorProp write
      SetColorProp;
    property ToolTipsBackColor: TColor index 163 read GetColorProp write
      SetColorProp;
    property OnExpand: TTListExpandEvent read FOnExpand write FOnExpand;
    property OnCollapse: TTListCollapseEvent read FOnCollapse write
      FOnCollapse;
    property OnPictureClick: TTListPictureClickEvent read FOnPictureClick
      write FOnPictureClick;
    property OnPictureDblClick: TTListPictureDblClickEvent read
      FOnPictureDblClick write FOnPictureDblClick;
    property OnClick: TTListClickEvent read FOnClick write FOnClick;
    property OnDblClick: TTListDblClickEvent read FOnDblClick write
      FOnDblClick;
    property OnKeyDown: TTListKeyDownEvent read FOnKeyDown write FOnKeyDown;
    property OnKeyUp: TTListKeyUpEvent read FOnKeyUp write FOnKeyUp;
    property OnKeyPress: TTListKeyPressEvent read FOnKeyPress write
      FOnKeyPress;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnEndDrag;
    property OnDragOver;
    property OnDragDropEx: TTListDragDropExEvent read FOnDragDropEx write
      FOnDragDropEx;
    property OnDragOverEx: TTListDragOverExEvent read FOnDragOverEx write
      FOnDragOverEx;
    property OnHScroll: TTListHScrollEvent read FOnHScroll write FOnHScroll;
    property OnVScroll: TTListVScrollEvent read FOnVScroll write FOnVScroll;
    property OnPlusMinusClick: TTListPlusMinusClickEvent read
      FOnPlusMinusClick write FOnPlusMinusClick;
    property OnPlusMinusDblClick: TTListPlusMinusDblClickEvent read
      FOnPlusMinusDblClick write FOnPlusMinusDblClick;
    property OnMarkClick: TTListMarkClickEvent read FOnMarkClick write
      FOnMarkClick;
    property OnMarkDblClick: TTListMarkDblClickEvent read FOnMarkDblClick
      write FOnMarkDblClick;
    property OnRequestEditing: TTListRequestEditingEvent read
      FOnRequestEditing write FOnRequestEditing;
    property OnAfterEditing: TTListAfterEditingEvent read FOnAfterEditing
      write FOnAfterEditing;
    property OnEditingKeyDown: TTListEditingKeyDownEvent read
      FOnEditingKeyDown write FOnEditingKeyDown;
    property OnEditingKeyUp: TTListEditingKeyUpEvent read FOnEditingKeyUp
      write FOnEditingKeyUp;
    property OnEditingKeyPress: TTListEditingKeyPressEvent read
      FOnEditingKeyPress write FOnEditingKeyPress;
  end;{ class }

 {declaration new types}
  TTreeBuffer   = LongInt;
  TTreeBookmark = LongInt;

const
  TListTrue  = 1;
  TListFalse = 0;

{ System & CoerceIndex properties constants }
  TLSYS_ENUM = $0;
  TLSYS_VIS = $1;
  TLSYS_LEVEL = $2;

{ PictureType property constants }
  TLPICTYPE_SOLVED = 0;
  TLPICTYPE_ROOT = 1;
  TLPICTYPE_OPEN = 2;
  TLPICTYPE_CLOSED = 3;
  TLPICTYPE_LEAF = 4;

{ InvStyle property constants }
  TLINVS_TEXTPIC = 0;
  TLINVS_PIC = 1;
  TLINVS_TEXT = 2;
  TLINVS_ALL = 3;
  TLINVS_TEXTPICEX = 4;
  TLINVS_TEXTBACK = 5;

{ ViewStyle property constants }
  TLVIEWS_PICTEXTLINKS = 0;
  TLVIEWS_PICTEXT = 1;
  TLVIEWS_PICLINKS = 2;
  TLVIEWS_PIC = 3;
  TLVIEWS_TEXTLINKS = 4;
  TLVIEWS_TEXT = 5;

{ MultiSelect property constants }
  TLMULSEL_NONE = 0;
  TLMULSEL_SIMPLE = 1;
  TLMULSEL_EXT = 2;

{ ItemType property constants }
  TLTYPE_NONE = $0;
  TLTYPE_INT = $494E;
  TLTYPE_LNG = $4C4E;
  TLTYPE_SNG = $534E;
  TLTYPE_STR = $5354;
  TLTYPE_PIC = $5450;

{ TListGetItemByXY function nType parameter constants }
  PTFIND_ITEM = 1;
  PTFIND_PIC = 2;

  TLGR_ITEM = $1;
  TLGR_PLUSMINUS = $2;
  TLGR_MARK = $4;
  TLGR_PICTURE = $8;
  TLGR_TEXT = $10;

{ Error codes constants }
  TLERR_BADINDENTATION = 20101;
  TLERR_WRONPARAMS = 20102; { Obselete; }
  TLERR_ITEMNOTFOUND = 20103;
  TLERR_PARENTNOTEXPANDED = 20104;
  TLERR_INITFAULT = 20105;
  TLERR_CANNOTSHOW = 20106;
  TLERR_INVALIDSEPARATOR = 20107;
  TLERR_INVALIDTREEBUFFER = 20108;
  TLERR_BIGPICSIZE = 20109;
  TLERR_NOTHINGTOCOPY = 20110;
  TLERR_TOMANYITEMS = 20111;
  TLERR_TOMANYSUBITEMS = 20112;
  TLERR_INVALIDARGS = 20113;

{ TriggerEvents property constants }
  TL_TRIGGER_INITIAL = 0;
  TL_TRIGGER_COLLAPSE_FOR_ALL = 1;

{ ItemEditText property  constants }
  TL_EDITTEXT_CANCEL = $0;
  TL_EDITTEXT_END = $1;
  TL_EDITTEXT_BEGIN = $2;

{ Clipboard property  constants }
  TL_CLP_COPY = 0;
  TL_CLP_COPYSUB = 1;
  TL_CLP_PASTEANDADD = 2;
  TL_CLP_PASTEANDINSERT = 3;

{ VScroll & HScroll ScrollCode parameter constants }
  TL_SB_BOTTOM = 7;
  TL_SB_ENDSCROLL = 8;
  TL_SB_LINEDOWN = 1;
  TL_SB_LINEUP = 0;
  TL_SB_PAGEDOWN = 3;
  TL_SB_PAGEUP = 2;
  TL_SB_THUMBPOSITION = 4;
  TL_SB_THUMBTRACK = 5;
  TL_SB_TOP = 6;

{ AftreEditing Options parameter constants }
  TL_EDCANCELLEDBY_USER = $0;
  TL_EDCANCELLEDBY_PROPERTY = $1;

{ RequestEditing Options parameter constants }
  TL_REQED_LEFT = $0;
  TL_REQED_CENTER = $1;
  TL_REQED_RIGHT = $2;
  TL_REQED_MULTILINE = $4;
  TL_REQED_UPPERCASE = $8;
  TL_REQED_LOWERCASE = $10;
  TL_REQED_PASSWORD = $20;
  TL_REQED_AUTOVSCROLL = $40;
  TL_REQED_AUTOHSCROLL = $80;
  TL_REQED_READONLY = $800;
  TL_REQED_STDCOLORS = $100;
  TL_REQED_BORDER = $200;
  TL_REQED_VSCROLLER = $400;
  TL_REQED_HSCROLLER = $2000;

{ Function Prototypes }
var

  TListFreeBuffer    : procedure(TreeBuffer: TTreeBuffer);
  TListIsValidBuffer : function(TreeBuffer: TTreeBuffer): Integer;

  TListTranslateIndex: function (TList: HCTL; nFromIndexMethod: Integer
                      ; nToIndexMethod: Integer; nFromIndex :Integer): Integer;

  TListGetItemByXY: function (TList: HCTL; nX: Integer; nY: Integer
           ; nType: Integer) : Integer;

{ New }
  TListIsClipboardFormatAvailable: function: Integer;

  TListGetItemRect: function (TList: HCTL; nIndex: Integer
           ; nOpts: Integer; Var pRect: TRect): Integer;

  TListSaveBuffer: function (TreeBuffer: TTreeBuffer; nFile: Integer): Integer;
  TListLoadBuffer: function (TreeBuffer: TTreeBuffer; nFile: Integer): Integer;

  TListCopyBuffer: function (TreeBuffer: TTreeBuffer): Integer;
  TListPasteBuffer: function (Var TreeBuffer: TTreeBuffer): Integer;

  TListIndexByBM: function (TList: HCTL; Var TreeBM: TTreeBookmark): Integer;
  TListIsValidBM: function (TList: HCTL; Var TreeBM: TTreeBookmark): Integer;

  TListFindItem: function (TList: HCTL; lpFindWhat: PChar; nFile: Integer; nFromIndex: Integer; nToIndex: Integer): Integer;

{----------------------------------------------------------------------------}
procedure TListAddItem(TList: TTList; Index: Integer; const Item: string);
procedure Register;

implementation

var
  hTListHandle: THandle;

{ Default form data for TTList }

{$IFDEF InitTTList}

const
  TTListInitLen = 182;

procedure TTListInitData; near; assembler;
asm
	DB	$24,$00,$43,$00,$00,$58,$00,$00,$5B,$01,$5E,$03,$00,$05,$54,$4C
	DB	$69,$73,$74,$01,$00,$00,$02,$00,$03,$09,$00,$10,$08,$00,$00,$80
	DB	$11,$05,$00,$00,$80,$12,$01,$14,$00,$15,$FF,$FF,$16,$00,$17,$00
	DB	$00,$18,$FF,$FF,$19,$00,$00,$1B,$00,$1C,$00,$1D,$14,$00,$00,$00
	DB	$1E,$0F,$00,$00,$00,$1F,$0F,$00,$00,$00,$33,$00,$35,$01,$5C,$3E
	DB	$00,$3F,$0E,$00,$00,$80,$40,$0D,$00,$00,$80,$42,$01,$00,$44,$08
	DB	$00,$00,$00,$47,$01,$48,$00,$00,$56,$00,$00,$59,$00,$00,$00,$00
	DB	$5A,$00,$00,$00,$00,$6D,$03,$6E,$00,$00,$6F,$02,$73,$00,$00,$00
	DB	$00,$74,$00,$00,$00,$00,$75,$01,$77,$00,$78,$00,$79,$00,$7A,$00
	DB	$7B,$00,$80,$14,$00,$00,$00,$81,$00,$00,$00,$00,$83,$00,$00,$8C
	DB	$00,$00,$8E,$00,$00,$8F,$00,$00,$92,$00,$98,$00,$00,$9A,$08,$00
	DB	$00,$00,$9B,$00,$00,$FF
end;

{$ENDIF}

{ TTList }

constructor TTList.Create(AOwner: TComponent);
begin
  FVBXFile := StrNew('TLISTPRO.VBX');
  FVBXClass := StrNew('TList');
  SetBounds(0, 0, 200, 200);
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csCaptureMouse, csClickEvents];
  TabStop := True;
  FVBXFlags := [vfArrows, vfLoadMessage];
  {$IFDEF InitTTList}
  FHForm := VBXCreateFormFile(TTListInitLen, @TTListInitData);
  {$ENDIF}
end;

procedure TTList.HandleVBXEvent(var Message: TWMVBXFireEvent);
begin
  case Message.VBXEvent^.EventIndex of
    0: DispatchCustomEvent(FOnExpand, Message, 0);
    1: DispatchCustomEvent(FOnCollapse, Message, 1);
    2: DispatchCustomEvent(FOnPictureClick, Message, 2);
    3: DispatchCustomEvent(FOnPictureDblClick, Message, 3);
    4: DispatchCustomEvent(FOnClick, Message, 4);
    5: DispatchCustomEvent(FOnDblClick, Message, 5);
    6: DispatchCustomEvent(FOnKeyDown, Message, 6);
    7: DispatchCustomEvent(FOnKeyUp, Message, 7);
    8: DispatchCustomEvent(FOnKeyPress, Message, 8);
    9: begin end; { VCL Handles OnGotFocus }
    10: begin end; { VCL Handles OnLostFocus }
    11: DispatchMouseEvent(OnMouseDown, Message);
    12: DispatchMouseMoveEvent(OnMouseMove, Message);
    13: DispatchMouseEvent(OnMouseUp, Message);
    14: begin end; { VCL Handles OnDragDrop }
    15: begin end; { VCL Handles OnDragOver }
    16: DispatchCustomEvent(FOnDragDropEx, Message, 16);
    17: DispatchCustomEvent(FOnDragOverEx, Message, 17);
    18: DispatchCustomEvent(FOnHScroll, Message, 18);
    19: DispatchCustomEvent(FOnVScroll, Message, 19);
    20: DispatchCustomEvent(FOnPlusMinusClick, Message, 20);
    21: DispatchCustomEvent(FOnPlusMinusDblClick, Message, 21);
    22: DispatchCustomEvent(FOnMarkClick, Message, 22);
    23: DispatchCustomEvent(FOnMarkDblClick, Message, 23);
    24: DispatchCustomEvent(FOnRequestEditing, Message, 24);
    25: DispatchCustomEvent(FOnAfterEditing, Message, 25);
    26: DispatchCustomEvent(FOnEditingKeyDown, Message, 26);
    27: DispatchCustomEvent(FOnEditingKeyUp, Message, 27);
    28: DispatchCustomEvent(FOnEditingKeyPress, Message, 28);
  end;
end;

procedure TTList.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  HandleBinaryProperty('PictureRoot_Data', Filer, 57);
  HandleBinaryProperty('PictureOpen_Data', Filer, 58);
  HandleBinaryProperty('PictureClosed_Data', Filer, 59);
  HandleBinaryProperty('PictureLeaf_Data', Filer, 60);
  HandleBinaryProperty('PictureInverted_Data', Filer, 61);
  HandleProperty('TriggerEvents', Filer, 90);
  HandleBinaryProperty('DragIconEx_Data', Filer, 93);
  HandleBinaryProperty('PicturePlus_Data', Filer, 106);
  HandleBinaryProperty('PictureMinus_Data', Filer, 107);
  HandleBinaryProperty('PictureMark_Data', Filer, 108);
end;

{ Designer registration }
procedure Register;
begin
  RegisterComponents('VBX', [TTList]);
end;

procedure TListAddItem(TList: TTList; Index: Integer; const Item: string);
var
  C : PChar;
begin
  C := StrAlloc(Length(Item) + 1);
  if C <> nil then
  begin
    StrPCopy(C, Item);
    VBXMethodAddItem(TList.Ctl, Index, C);
    StrDispose(C);
  end;
end;

procedure UnloadTList; far;
begin
  FreeLibrary(hTListHandle);
end;

begin
  hTListHandle := LoadLibrary('tlistpro.vbx');
  if hTListHandle >= 32 then
  begin
    AddExitProc(UnloadTList);

    @TListFreeBuffer    := GetProcAddress(hTListHandle, 'TListFreeBuffer');
    @TListIsValidBuffer := GetProcAddress(hTListHandle, 'TListIsValidBuffer');
    @TListTranslateIndex:= GetProcAddress(hTListHandle, 'TListTranslateIndex');
    @TListGetItemByXY   := GetProcAddress(hTListHandle, 'TListGetItemByXY');
    @TListIsClipboardFormatAvailable :=
                           GetProcAddress(hTListHandle
                           , 'TListIsClipboardFormatAvailable');
    @TListGetItemRect   := GetProcAddress(hTListHandle, 'TListGetItemRect');
    @TListSaveBuffer    := GetProcAddress(hTListHandle, 'TListSaveBuffer');
    @TListLoadBuffer    := GetProcAddress(hTListHandle, 'TListLoadBuffer');
    @TListCopyBuffer    := GetProcAddress(hTListHandle, 'TListCopyBuffer');
    @TListPasteBuffer   := GetProcAddress(hTListHandle, 'TListPasteBuffer');
    @TListIndexByBM     := GetProcAddress(hTListHandle, 'TListIndexByBM');
    @TListIsValidBM     := GetProcAddress(hTListHandle, 'TListIsValidBM');
    @TListFindItem      := GetProcAddress(hTListHandle, 'TListFindItem');

  end
  else
  begin
    MessageBox(0, 'Cannot load TLISTPRO.VBX', nil
    , mb_IconStop or mb_OK or mb_SystemModal);
    Halt(0);
  end;
end.

end.
